function FDMa_make_graph(A,graphstyle,filename,filetype,edge_weight_factor,edge_labels,node_labels,node_colors,node_positions,node_slot)
% creates graph from adjacency Matrix A with Output .dot and .filetype     
% 
% leave options empty with ''
%
% 'A' (required)  adjacency matrix of (weighted) (directed) graph
% 'graphstyle' (optional, default is neato)
%         dot    filter for drawing directed graphs
%         neato  filter for drawing undirected graphs
%         twopi  filter for radial layouts of graphs
%         circo  filter for circular layout of graphs
%         fdp    filter for drawing undirected graphs
% 'filename' (optional)
% 'filetype' (optional) 
%         allowed filetypes: 
%         canon cmap cmapx dia dot fig gd gd2 gif gtk hpgl 
%         imap ismap jpeg jpg mif mp pcl pdf pic plain plain-ext 
%         png ps ps2 svg svgz vrml vtx wbmp xdot xlib
% 'edge_weight_factor' weighting of edges (optional, default 5, no weighting 0)
% 'edge_labels' (optional, default is off)
%         if you put an array or a matrix (eg. A) here, the values are used as edge labels
% 'node_labels'
%         example: {'"node 1"','"node 2"','"node 3"'}
% 'node_colors' 
%         example: {'"black"','"green"','"yellow"'}
%         number of colors should be >= length(A)
%         see e.g. http://www.graphviz.org/doc/info/colors.html
% 'node_positions'
%         example: {'0,0!','0,2!','2,0!'}
%         number of coordinates should be >= length(A)
% 'node_slot' additional styles for single nodes
%         example:
%         {'','','fontcolor="organge"'}
%         set fontcolor of node 3 orange
% Example: 
%         A=[0 1 1;1 0 0;4 0 0]; make_graph(A) or make_graph(A,'circo')
%
% requitres GraphViz (AT&T) http://www.graphviz.org/
%
% written by Jakob Nawrath 03.2008 nawrath@fdm.uni-freiburg.de

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Global Layout. Change here or pass arguments to make_graph when 
% you want to change the style for single nodes or edges
% for customization see http://www.graphviz.org/doc/info/attrs.html
% pattern: -(G|N|E)name=value
global_graph_style='-Gcolor=black';
%global_node_style='-Ncolor=blue -Nstyle=filled -Nfontcolor=white';
%global_node_style='-Ncolor=blue  -Nfontcolor=white';
global_node_style='-Ncolor=blue  -Nfontcolor=black';
%global_edge_style='-Earrowhead=normal';
global_edge_style='';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,m] = size(A);
if n ~= m, warning('not a square adjacency matrix!'); end
if ~isequal(diag(A),zeros(n,1)), warning('Self-loops in adjacency matrix!');end

%% filetype: default output filetype is 'ps'
if exist('filetype','var')==0 || isempty(filetype); filetype='ps'; end

%% filename: if not specified, output is set to tmp.ps and tmp.dot
if exist('filename','var')==0 || isempty(filename); savefile=['tmp.' filetype]; dotout='tmp.dot';
else savefile=[filename '.' filetype];dotout=[filename '.dot'];
end

%% graphstyle: default is 'dot'
if exist('graphstyle','var')==0 || isempty(graphstyle); graphstyle='dot'; end

%% NODE options
% node labels
if exist('node_labels','var')==0 || isempty(node_labels); node_labels=''; end

% node colors
if exist('node_colors','var')==0 || isempty(node_colors);node_colors='';
else
    node_colors=node_colors(1:n);
end

% node positions
if exist('node_positions','var')==0 || isempty(node_positions); node_positions='';
else graphstyle='neato'; %warning(['Graph Layout was set to ' graphstyle]);
end

% node slot
if exist('node_slot','var')==0; node_slot='';end

%% process NODEs 
node_attributes=cell(1,n);
for node = 1:n
    if ~isempty(node_labels);node_attributes{1,node}=[node_attributes{1,node} 'label = ' node_labels{node} ', '];end
    if ~isempty(node_colors);node_attributes{1,node}=[node_attributes{1,node} 'color = ' node_colors{node} ', '];end
    if ~isempty(node_positions);node_attributes{1,node}=[node_attributes{1,node} 'pos = "' node_positions{node} '", '];end
    if ~isempty(node_slot);node_attributes{1,node}=[node_attributes{1,node} node_slot{node}];end
    % put more node attributes here
    % ...
    if ~isempty(node_attributes{1,node});node_attributes{1,node}=['[' node_attributes{1,node} ']'];else node_attributes{1,node}='';end
    node_attributes{1,node}=strrep(node_attributes{1,node},', ]',']');
end

%% EDGE options
% edge line weighting 
if ~exist('edge_weight_factor','var') || isempty(edge_weight_factor),edge_weight_factor=5;end
for i=1:n;
    for j=1:m;
        edge_linewidth{i,j}=num2str(A(i,j));
        if edge_weight_factor~=0,
            edge_linewidth{i,j}=num2str(edge_weight_factor*A(i,j));
        else
            edge_linewidth{i,j}=num2str(heaviside(A(i,j)));
        end;
    end
end
   
% edge labels
if ~exist('edge_labels','var'),edge_label='';
else
    if isempty(edge_labels),edge_label='';
    else
        if isvector(edge_labels),
            for i=1:n;
                for j=1:m;
                        edge_label{i,j}=num2str(edge_labels(i,j));
                end
            end
        end
        if iscell(edge_labels),
            edge_label=edge_labels
        end
    end
end

%% process EDGEs
edge_attrib_string = [];
if ~isempty(edge_weight_factor),edge_attrib_string = [edge_attrib_string,'style="setlinewidth(%s)", '];end
if ~isempty(edge_label),edge_attrib_string = [edge_attrib_string,'label="%s", '];end
% put more edge attributes here
% ...
if ~isempty(edge_attrib_string),edge_attrib_string=['[' edge_attrib_string ']'];else edge_attrib_string='';end
edge_attrib_string=strrep(edge_attrib_string,', ]',']');

%% OUTPUT
% make dot file
adj_to_dot(A, 'filename', dotout, 'edge_attrib_string', edge_attrib_string, 'edge_linewidth', edge_linewidth, 'edge_label', edge_label, 'node_attributes', node_attributes);             

% request version to check Grapviz is there
%status = system([graphstyle ' -V']);
%if status == 1,  warning([graphstyle ' not accessible']); end

% put Graphviz options here
command=char([graphstyle ' ' global_graph_style ' ' global_node_style ' ' global_edge_style ' -T' filetype ' ' dotout ' -o ' savefile ' -o']);

% pass command to DOS or LINUX 
[status, result] = system(command);
%display(['wrote ' dotout ' and ' savefile ' to ' pwd]);


function adj_to_dot(adj, varargin)
%% adj_to_dot(adj, VARARGIN)  
% Creates a GraphViz (AT&T) format file representing 
% a graph given by an adjacency matrix.
% Optional arguments should be passed as name/value pairs [default]
%
% 'filename'   -  if omitted, writes to 'tmp.dot'
% 'edge_attrib_string'
% 'edge_linewidth'
% 'edge_label'
% 'node_attributes'
% 'width'      -  width in inches [10]
% 'height'     -  height in inches [10]
% 'leftright'  -  1 means layout left-to-right, 0 means top-to-bottom [0]
% 'directed'   -  1 means use directed edges, 0 means undirected [1]
%
% Original by Dr. Leon Peshkin, Jan 2004 pesha@ai.mit.edu/~pesha
% Modifications by Jakob Nawrath, 03.2008 nawrath@fdm.uni-freiburg.de

% set default args
edge_label = [];   
width = 10;        height = 10;
filename = 'tmp.dot';
           
%% get optional args
for i = 1:2:nargin-1              
    switch varargin{i}
        case 'filename', filename = varargin{i+1};
        case 'edge_attrib_string', edge_attrib_string = varargin{i+1};
        case 'edge_label', edge_label = varargin{i+1};
        case 'edge_linewidth', edge_linewidth = varargin{i+1};                        
        case 'node_attributes', node_attributes = varargin{i+1};
        case 'width', width = varargin{i+1};
        case 'height', height = varargin{i+1};            
    end
end
fid = fopen(filename, 'w');

%% prepare .dot file
%fprintf(fid, 'digraph G {\n');
fprintf(fid, 'graph G {\n');
fprintf(fid, 'center = 1;\n');
%%%% MJ: writes the filename below the graph
fprintf(fid, 'label= "%s";', filename);

fprintf(fid, 'size=\"%d,%d\";\n', width, height);

Nnds = length(adj);

%% process NODEs 
for node = 1:Nnds               
    if isempty(node_attributes)
        fprintf(fid, '%d;\n', node);
    else
        fprintf(fid, '%d%s;\n', node,  node_attributes{node});
    end
end

%% prepare EDGEs
edgetxt = '--'; 
if isempty(edge_attrib_string)
    edge_attribute = '';
else
    edge_attribute = edge_attrib_string;
end

%% edge format
edgeformat = strcat(['%d ',edgetxt,' %d ',edge_attribute,';\n']);

%% process EDGEs
for node1 = 1:Nnds              
        edges = find(adj(node1,:));         
    for node2 = edges
        if isempty(edge_label)     
            fprintf(fid, edgeformat, node1, node2, edge_linewidth{node1,node2});  
        else
            fprintf(fid, edgeformat, node1, node2, edge_linewidth{node1,node2}, edge_label{node1,node2});
        end
    end
end
fprintf(fid, '}'); 
fclose(fid);
